@@ -3,7 +3,7 @@
 **
 ** <file/class description>
 **
-** Copyright (C) 2002-2003 Steve Lhomme.  All rights reserved.
+** Copyright (C) 2002-2004 Steve Lhomme.  All rights reserved.
 **
 ** This file may be distributed under the terms of the Q Public License
 ** as defined by Trolltech AS of Norway and appearing in the file
@@ -30,7 +30,7 @@
 
 /*!
 	\file
-	\version \$Id: KaxBlock.cpp,v 1.18 2003/12/03 16:17:21 cyrius Exp $
+	\version \$Id: KaxBlock.cpp,v 1.24 2004/01/15 20:47:37 cyrius Exp $
 	\author Steve Lhomme     <robux4 @ users.sf.net>
 	\author Julien Coloos    <suiryc @ users.sf.net>
 */
@@ -38,7 +38,6 @@
 
 //#include <streams.h>
 
-#include "StdInclude.h"
 #include "matroska/KaxBlock.h"
 #include "matroska/KaxContexts.h"
 #include "matroska/KaxBlockData.h"
@@ -46,18 +45,27 @@
 
 START_LIBMATROSKA_NAMESPACE
 
+#if MATROSKA_VERSION == 1
+const EbmlSemantic KaxBlockGroup_ContextList[5] =
+#else // MATROSKA_VERSION
 const EbmlSemantic KaxBlockGroup_ContextList[8] =
+#endif // MATROSKA_VERSION
 {
 	EbmlSemantic(true,  true,  KaxBlock::ClassInfos),
+#if MATROSKA_VERSION >= 2
 	EbmlSemantic(false, true,  KaxBlockVirtual::ClassInfos),
+#endif // MATROSKA_VERSION
 	EbmlSemantic(false, true,  KaxBlockDuration::ClassInfos),
 	EbmlSemantic(false, true,  KaxSlices::ClassInfos),
 	EbmlSemantic(true,  true,  KaxReferencePriority::ClassInfos),
 	EbmlSemantic(false, false, KaxReferenceBlock::ClassInfos),
+#if MATROSKA_VERSION >= 2
 	EbmlSemantic(false, true,  KaxReferenceVirtual::ClassInfos),
 	EbmlSemantic(false, true,  KaxBlockAdditions::ClassInfos),
+#endif // MATROSKA_VERSION
 };
 
+#if MATROSKA_VERSION >= 2
 const EbmlSemantic KaxBlockAdditions_ContextList[1] =
 {
 	EbmlSemantic(true,  false,  KaxBlockMore::ClassInfos)
@@ -68,33 +76,40 @@ const EbmlSemantic KaxBlockMore_ContextList[2] =
 	EbmlSemantic(true,  true,  KaxBlockAddID::ClassInfos),
 	EbmlSemantic(true,  true,  KaxBlockAdditional::ClassInfos)
 };
+#endif // MATROSKA_VERSION
 
 EbmlId KaxBlockGroup_TheId     (0xA0, 1);
 EbmlId KaxBlock_TheId          (0xA1, 1);
-EbmlId KaxBlockVirtual_TheId   (0xA2, 1);
 EbmlId KaxBlockDuration_TheId  (0x9B, 1);
+#if MATROSKA_VERSION >= 2
+EbmlId KaxBlockVirtual_TheId   (0xA2, 1);
 EbmlId KaxBlockAdditions_TheId (0x75A1, 2);
 EbmlId KaxBlockMore_TheId      (0xA6, 1);
 EbmlId KaxBlockAddID_TheId     (0xEE, 1);
 EbmlId KaxBlockAdditional_TheId(0xA5, 1);
+#endif // MATROSKA_VERSION
 
 const EbmlSemanticContext KaxBlockGroup_Context = EbmlSemanticContext(countof(KaxBlockGroup_ContextList), KaxBlockGroup_ContextList, &KaxCluster_Context, *GetKaxGlobal_Context, &KaxBlockGroup::ClassInfos);
 const EbmlSemanticContext KaxBlock_Context = EbmlSemanticContext(0, NULL, &KaxBlockGroup_Context, *GetKaxGlobal_Context, &KaxBlock::ClassInfos);
-const EbmlSemanticContext KaxBlockVirtual_Context = EbmlSemanticContext(0, NULL, &KaxBlockGroup_Context, *GetKaxGlobal_Context, &KaxBlockVirtual::ClassInfos);
 const EbmlSemanticContext KaxBlockDuration_Context = EbmlSemanticContext(0, NULL, &KaxBlockGroup_Context, *GetKaxGlobal_Context, &KaxBlockDuration::ClassInfos);
+#if MATROSKA_VERSION >= 2
+const EbmlSemanticContext KaxBlockVirtual_Context = EbmlSemanticContext(0, NULL, &KaxBlockGroup_Context, *GetKaxGlobal_Context, &KaxBlockVirtual::ClassInfos);
 const EbmlSemanticContext KaxBlockAdditions_Context = EbmlSemanticContext(countof(KaxBlockAdditions_ContextList), KaxBlockAdditions_ContextList, &KaxBlockGroup_Context, *GetKaxGlobal_Context, &KaxBlockAdditions::ClassInfos);
 const EbmlSemanticContext KaxBlockMore_Context = EbmlSemanticContext(countof(KaxBlockMore_ContextList), KaxBlockMore_ContextList, &KaxBlockAdditions_Context, *GetKaxGlobal_Context, &KaxBlockMore::ClassInfos);
 const EbmlSemanticContext KaxBlockAddID_Context = EbmlSemanticContext(0, NULL, &KaxBlockMore_Context, *GetKaxGlobal_Context, &KaxBlockAddID::ClassInfos);
 const EbmlSemanticContext KaxBlockAdditional_Context = EbmlSemanticContext(0, NULL, &KaxBlockMore_Context, *GetKaxGlobal_Context, &KaxBlockAdditional::ClassInfos);
+#endif // MATROSKA_VERSION
 
 const EbmlCallbacks KaxBlockGroup::ClassInfos(KaxBlockGroup::Create, KaxBlockGroup_TheId, "BlockGroup", KaxBlockGroup_Context);
 const EbmlCallbacks KaxBlock::ClassInfos(KaxBlock::Create, KaxBlock_TheId, "Block", KaxBlock_Context);
-const EbmlCallbacks KaxBlockVirtual::ClassInfos(KaxBlockVirtual::Create, KaxBlockVirtual_TheId, "BlockVirtual", KaxBlockVirtual_Context);
 const EbmlCallbacks KaxBlockDuration::ClassInfos(KaxBlockDuration::Create, KaxBlockDuration_TheId, "BlockDuration", KaxBlockDuration_Context);
+#if MATROSKA_VERSION >= 2
+const EbmlCallbacks KaxBlockVirtual::ClassInfos(KaxBlockVirtual::Create, KaxBlockVirtual_TheId, "BlockVirtual", KaxBlockVirtual_Context);
 const EbmlCallbacks KaxBlockAdditions::ClassInfos(KaxBlockAdditions::Create, KaxBlockAdditions_TheId, "BlockAdditions", KaxBlockAdditions_Context);
 const EbmlCallbacks KaxBlockMore::ClassInfos(KaxBlockMore::Create, KaxBlockMore_TheId, "BlockMore", KaxBlockMore_Context);
 const EbmlCallbacks KaxBlockAddID::ClassInfos(KaxBlockAddID::Create, KaxBlockAddID_TheId, "BlockAddID", KaxBlockAddID_Context);
 const EbmlCallbacks KaxBlockAdditional::ClassInfos(KaxBlockAdditional::Create, KaxBlockAdditional_TheId, "BlockAdditional", KaxBlockAdditional_Context);
+#endif // MATROSKA_VERSION
 
 DataBuffer * DataBuffer::Clone()
 {
@@ -155,6 +170,7 @@ KaxBlockGroup::KaxBlockGroup()
  ,ParentTrack(NULL)
 {}
 
+#if MATROSKA_VERSION >= 2
 KaxBlockAdditions::KaxBlockAdditions()
  :EbmlMaster(KaxBlockAdditions_Context)
 {}
@@ -162,6 +178,7 @@ KaxBlockAdditions::KaxBlockAdditions()
 KaxBlockMore::KaxBlockMore()
  :EbmlMaster(KaxBlockMore_Context)
 {}
+#endif // MATROSKA_VERSION
 
 /*!
 	\todo handle flags
@@ -274,6 +291,7 @@ uint64 KaxBlock::UpdateSize(bool bSaveDefault, bool bForceRender)
 	return Size;
 }
 
+#if MATROSKA_VERSION >= 2
 KaxBlockVirtual::KaxBlockVirtual(const KaxBlockVirtual & ElementToClone)
  :EbmlBinary(ElementToClone)
  ,Timecode(ElementToClone.Timecode)
@@ -308,6 +326,7 @@ uint64 KaxBlockVirtual::UpdateSize(bool bSaveDefault, bool bForceRender)
 
 	return Size;
 }
+#endif // MATROSKA_VERSION
 
 /*!
 	\todo more optimisation is possible (render the Block head and don't copy the buffer in memory, care should be taken with the allocation of Data)
@@ -648,15 +667,14 @@ uint64 KaxBlock::ReadData(IOCallback & input, ScopeMode ReadFully)
 					// get the size of the frame
 					FrameSize = 0;
 					do {
+						Result += input.read(_TempHead, 1);
 						FrameSize += uint8(_TempHead[0]);
 						LastBufferSize--;
 
 						FirstFrameLocation++;
-						Result += input.read(_TempHead, 1);
 					} while (_TempHead[0] == 0xFF);
 
 					FirstFrameLocation++;
-					Result += input.read(_TempHead, 1);
 					SizeList[Index] = FrameSize;
 					LastBufferSize -= FrameSize;
 				}
