@@ -3,7 +3,7 @@
 **
 ** <file/class description>
 **
-** Copyright (C) 2002-2003 Steve Lhomme.  All rights reserved.
+** Copyright (C) 2002-2004 Steve Lhomme.  All rights reserved.
 **
 ** This file may be distributed under the terms of the Q Public License
 ** as defined by Trolltech AS of Norway and appearing in the file
@@ -30,7 +30,7 @@
 
 /*!
     \file
-    \version \$Id: test8.cpp,v 1.3 2003/11/03 22:17:35 robux4 Exp $
+    \version \$Id: test8.cpp,v 1.8 2004/01/17 16:33:21 robux4 Exp $
     \brief Test reading of the Header
     \author Steve Lhomme     <robux4 @ users.sf.net>
 */
@@ -90,8 +90,8 @@ int main(int argc, char **argv)
 	EbmlElement * ElementLevel0;
 	EbmlElement * ElementLevel1;
 	EbmlElement * ElementLevel2;
-	EbmlElement * ElementLevel3;
-	EbmlElement * ElementLevel4;
+	EbmlElement * ElementLevel3 = NULL;
+	EbmlElement * ElementLevel4 = NULL;
 
 ///	MuxedFile.ReadHead();
 	// find the EBML head in the file
@@ -131,7 +131,7 @@ int main(int argc, char **argv)
 //			MuxedFile.ReadTracks();
 //			MuxedFile.ReadCodec();
 			// scan the file for a Tracks element (all previous Level1 elements are discarded)
-			ElementLevel1 = aStream.FindNextElement(ElementLevel0->Generic().Context, UpperElementLevel, ElementLevel0->ElementSize(), bAllowDummy);
+			ElementLevel1 = aStream.FindNextElement(ElementLevel0->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 
 			while (ElementLevel1 != NULL) {
 				if (UpperElementLevel > 0) {
@@ -149,7 +149,7 @@ int main(int argc, char **argv)
 					printf("\n- Segment Tracks found\n");
 					// handle the data in Tracks here.
 					// poll for new tracks and handle them
-					ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, ElementLevel1->ElementSize(), bAllowDummy);
+					ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 
 					while (ElementLevel2 != NULL) {
 						if (UpperElementLevel > 0) {
@@ -162,7 +162,7 @@ int main(int argc, char **argv)
 						if (EbmlId(*ElementLevel2) == KaxTrackEntry::ClassInfos.GlobalId) {
 							printf("* Found a track\n");
 
-							ElementLevel3 = aStream.FindNextElement(ElementLevel2->Generic().Context, UpperElementLevel, ElementLevel2->ElementSize(), bAllowDummy);
+							ElementLevel3 = aStream.FindNextElement(ElementLevel2->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 							while (ElementLevel3 != NULL) {
 								if (UpperElementLevel > 0) {
 									break;
@@ -201,6 +201,9 @@ int main(int argc, char **argv)
 									printf("\n");
 								}
 
+								else if (EbmlId(*ElementLevel3) == KaxTrackFlagLacing::ClassInfos.GlobalId) {
+									printf("Flag Lacing\n");
+								}
 								else if (EbmlId(*ElementLevel3) == KaxCodecID::ClassInfos.GlobalId) {
 									KaxCodecID & CodecID = *static_cast<KaxCodecID*>(ElementLevel3);
 									CodecID.ReadData(aStream.I_O());
@@ -218,7 +221,7 @@ int main(int argc, char **argv)
 									ElementLevel3->SkipData(aStream, ElementLevel3->Generic().Context);
 									delete ElementLevel3;
 
-									ElementLevel3 = aStream.FindNextElement(ElementLevel2->Generic().Context, UpperElementLevel, ElementLevel2->ElementSize(), bAllowDummy);
+									ElementLevel3 = aStream.FindNextElement(ElementLevel2->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 								}
 							}
 						}
@@ -232,7 +235,7 @@ int main(int argc, char **argv)
 							ElementLevel2->SkipData(aStream, ElementLevel2->Generic().Context);
 							delete ElementLevel2;
 					
-							ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, ElementLevel1->ElementSize(), bAllowDummy);
+							ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 						}
 					}
 				}
@@ -246,12 +249,12 @@ int main(int argc, char **argv)
 					aStream.I_O().setFilePointer(79, seek_current);
 					printf("Seeked at position 0x%X in the file", aStream.I_O().getFilePointer());
 					int LowLevel = 1;
-					ElementLevel2 = aStream.FindNextElement(KaxSegment::ClassInfos.Context, LowLevel, ElementLevel0->ElementSize(), false); // search up to one level lower
+					ElementLevel2 = aStream.FindNextElement(KaxSegment::ClassInfos.Context, LowLevel, 0, false); // search up to one level lower
 					UpperElementLevel = LowLevel;
 #else // JUMP_TEST
 					// read the data we care about in matroska
 					/// \todo There should be a way to get the default values of the elements not defined
-					ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, ElementLevel1->ElementSize(), bAllowDummy);
+					ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 					while (ElementLevel2 != NULL) {
 						if (UpperElementLevel > 0) {
 							break;
@@ -298,7 +301,7 @@ int main(int argc, char **argv)
 							ElementLevel2->SkipData(aStream, ElementLevel2->Generic().Context);
 							delete ElementLevel2;
 
-							ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, ElementLevel1->ElementSize(), bAllowDummy);
+							ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 						}
 					}
 #endif // JUMP_TEST
@@ -311,7 +314,7 @@ int main(int argc, char **argv)
 					uint32 ClusterTimecode;
 					EbmlCrc32 *pChecksum = NULL;
 					uint32 SizeInCrc;
-					uint64 CrcPositionStart;
+					uint64 CrcPositionStart = 0;
 
 #ifdef MEMORY_READ // read the Cluster in memory and then extract elements from memory
 					SegmentCluster->Read(aStream, KaxCluster::ClassInfos.Context, UpperElementLevel, ElementLevel2, bAllowDummy);
@@ -328,7 +331,7 @@ int main(int argc, char **argv)
 					}
 #else // not MEMORY_READ
 					// read blocks and discard the ones we don't care about
-					ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, ElementLevel1->ElementSize(), bAllowDummy);
+					ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 					while (ElementLevel2 != NULL) {
 						if (UpperElementLevel > 0) {
 							break;
@@ -344,9 +347,9 @@ int main(int argc, char **argv)
 							SegmentCluster->InitTimecode(ClusterTimecode, TimecodeScale);
 						} else  if (EbmlId(*ElementLevel2) == KaxBlockGroup::ClassInfos.GlobalId) {
 							printf("Block Group found\n");
+#ifdef TEST_BLOCKGROUP_READ
 							KaxBlockGroup & aBlockGroup = *static_cast<KaxBlockGroup*>(ElementLevel2);
 //							aBlockGroup.ClearElement();
-#ifdef TEST_BLOCKGROUP_READ
 							// Extract the valuable data from the Block
 
 							aBlockGroup.Read(aStream, KaxBlockGroup::ClassInfos.Context, UpperElementLevel, ElementLevel3, bAllowDummy);
@@ -369,7 +372,7 @@ int main(int argc, char **argv)
 #else // TEST_BLOCKGROUP_READ
 							// read the data we care about in matroska
 							/// \todo There should be a way to get the default values of the elements not defined
-							ElementLevel3 = aStream.FindNextElement(ElementLevel2->Generic().Context, UpperElementLevel, ElementLevel2->ElementSize(), bAllowDummy);
+							ElementLevel3 = aStream.FindNextElement(ElementLevel2->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 							while (ElementLevel3 != NULL) {
 								if (UpperElementLevel > 0) {
 									break;
@@ -393,16 +396,18 @@ int main(int argc, char **argv)
 									}
 #endif // NO_DISPLAY_DATA
 //									printf("Codec ID   : %s\n", &binary(CodecID)); // strings for the moment (example)
+#if MATROSKA_VERSION >= 2
 								} else if (EbmlId(*ElementLevel3) == KaxBlockVirtual::ClassInfos.GlobalId) {
 									printf(" Virtual Block\n");
+								} else if (EbmlId(*ElementLevel3) == KaxReferenceVirtual::ClassInfos.GlobalId) {
+									printf("  virtual Reference\n");
+#endif // MATROSKA_VERSION
 								} else if (EbmlId(*ElementLevel3) == KaxReferencePriority::ClassInfos.GlobalId) {
 									printf("  Reference priority\n");
 								} else if (EbmlId(*ElementLevel3) == KaxReferenceBlock::ClassInfos.GlobalId) {
 									KaxReferenceBlock & RefTime = *static_cast<KaxReferenceBlock*>(ElementLevel3);
 									RefTime.ReadData(aStream.I_O());
 									printf("  Reference frame at scaled (%d) timecode %ld\n", int32(RefTime), int32(int64(RefTime) * TimecodeScale));
-								} else if (EbmlId(*ElementLevel3) == KaxReferenceVirtual::ClassInfos.GlobalId) {
-									printf("  virtual Reference\n");
 								} else if (EbmlId(*ElementLevel3) == KaxBlockDuration::ClassInfos.GlobalId) {
 									KaxBlockDuration & BlockDuration = *static_cast<KaxBlockDuration*>(ElementLevel3);
 									BlockDuration.ReadData(aStream.I_O());
@@ -417,7 +422,7 @@ int main(int argc, char **argv)
 								} else {
 									ElementLevel3->SkipData(aStream, ElementLevel3->Generic().Context);
 
-									ElementLevel3 = aStream.FindNextElement(ElementLevel2->Generic().Context, UpperElementLevel, ElementLevel2->ElementSize(), bAllowDummy);
+									ElementLevel3 = aStream.FindNextElement(ElementLevel2->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 								}
 							}
 #endif // TEST_BLOCKGROUP_READ
@@ -441,7 +446,7 @@ int main(int argc, char **argv)
 							if (ElementLevel2 != pChecksum)
 								delete ElementLevel2;
 
-							ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, ElementLevel1->ElementSize(), bAllowDummy);
+							ElementLevel2 = aStream.FindNextElement(ElementLevel1->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 						}
 					}
 #endif // not MEMORY_READ
@@ -500,6 +505,7 @@ int main(int argc, char **argv)
 										} else if (CuePos[Index2]->Generic().GlobalId == KaxCueClusterPosition::ClassInfos.GlobalId) {
 											KaxCueClusterPosition & CuePoss = *static_cast<KaxCueClusterPosition *>(CuePos[Index2]);
 											printf("   Cluster position %d\n", uint64(CuePoss));
+#if MATROSKA_VERSION >= 2
 										} else if (CuePos[Index2]->Generic().GlobalId == KaxCueReference::ClassInfos.GlobalId) {
 											KaxCueReference & CueRefs = *static_cast<KaxCueReference *>(CuePos[Index2]);
 											printf("   Reference\n");
@@ -516,6 +522,7 @@ int main(int argc, char **argv)
 													printf("    - found %s\n", CueRefs[Index3]->Generic().DebugName);
 												}
 											}
+#endif // MATROSKA_VERSION
 										} else {
 											printf("   - found %s\n", CuePos[Index2]->Generic().DebugName);
 										}
@@ -549,7 +556,7 @@ int main(int argc, char **argv)
 								if (SeekPoint[Index1]->Generic().GlobalId == KaxSeekID::ClassInfos.GlobalId) {
 									KaxSeekID * SeekID = static_cast<KaxSeekID *>(SeekPoint[Index1]);
 									printf("    Seek ID ", SeekID->GetBuffer());
-									for (int i=0; i<SeekID->GetSize(); i++) {
+									for (unsigned int i=0; i<SeekID->GetSize(); i++) {
 										printf("%02X", SeekID->GetBuffer()[i]);
 									}
 									printf("\n");
@@ -632,7 +639,7 @@ int main(int argc, char **argv)
 							KaxTag & TagElt = *static_cast<KaxTag *>((*AllTags)[Index0]);
 							unsigned int Index1;
 							for (Index1 = 0; Index1<TagElt.ListSize() ;Index1++) {
-								bool bRemoved = false, bRemovedDone = true;
+// 								bool bRemoved = false, bRemovedDone = true;
 								if (TagElt[Index1]->Generic().GlobalId == KaxTagTargets::ClassInfos.GlobalId) {
 									printf("    Targets\n");
 									KaxTagTargets & Targets = *static_cast<KaxTagTargets *>(TagElt[Index1]);
@@ -748,7 +755,7 @@ int main(int argc, char **argv)
 					ElementLevel1->SkipData(aStream, ElementLevel1->Generic().Context);
 					delete ElementLevel1;
 			
-					ElementLevel1 = aStream.FindNextElement(ElementLevel0->Generic().Context, UpperElementLevel, ElementLevel0->ElementSize(), bAllowDummy);
+					ElementLevel1 = aStream.FindNextElement(ElementLevel0->Generic().Context, UpperElementLevel, 0, bAllowDummy);
 				}
 			}
 		}
